
/* ISA IO port method for accessing Hardware Monitor Chip */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <fcntl.h>

#ifndef LINUX	/* FreeBSD */
#include <machine/cpufunc.h>
#define OUTb(x,y)	outb((x),(y))
#define WAIT	outb(0xEB,0x00)
#else			/* LINUX */
#include <unistd.h>
#include <asm/io.h>
#define OUTb(x,y)	outb((y),(x))
#define WAIT	outb(0x00,0xEB)
#endif

#include "methods.h"
static int iofl;

static int OpenIO()
{
#ifndef LINUX	/* FreeBSD */
  if((iofl=open("/dev/io",000))<0){
	return -1;
  }
#else			/* LINUX */
  ioperm(WBIO1,1,1);
  ioperm(WBIO2,1,1);
#endif
  return 0;
}
static int CloseIO()
{
#ifndef LINUX	/* FreeBSD */
  close(iofl);
#else			/* LINUX */
  ioperm(WBIO1,1,0);
  ioperm(WBIO2,1,0);
#endif
}

static WriteByte(int addr,int value)
{
  OUTb(WBIO1,addr); WAIT;
  OUTb(WBIO2,value); WAIT;
}

static int ReadByte(int addr)
{
  int ret;
  OUTb(WBIO1,addr); WAIT;
  ret = inb(WBIO2); WAIT;
  return (ret & 0xFF);
}

static int ReadTemp1()
{
	int n;
	WriteByte(0x4E,0x01); /* changing to bank 1 */
	n = ReadByte(0x50) + ((ReadByte(0x51) & 0xFF) << 8);
	WriteByte(0x4E,0);	/* returning to bank 0 */
	return n;
}

static int ReadTemp2()
{
	int n;
	WriteByte(0x4E,0x02); /* changing to bank 2 */
	n = ReadByte(0x50) + ((ReadByte(0x51) & 0xFF) << 8);
	WriteByte(0x4E,0);	/* returning to bank 0 */
	return n;
}

struct lm_methods method_isa={
  OpenIO,CloseIO,ReadByte,WriteByte,ReadTemp1,ReadTemp2
};
  
