
/* SMBus method for accessing Hardware Monitor Chip */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* SMBus Base Address, global */
int smb_base;

#if !defined(LINUX) && defined(HAVE_SMBUS) && defined(SMBUS_IOCTL)

/* Using SMBus ioctl, need "xxxpm" driver !! */
#include "getMB-smb_ioctl.c"

#else		/* SMBus direct access routines */

#include <fcntl.h>
#include <sys/types.h>

#ifndef LINUX
#include <machine/cpufunc.h>
#else
#include <unistd.h>
#include <asm/io.h>
#endif

#include "methods.h"
#include "smb_io.c"

static int iofl;

static int OpenIO()
{
	if (smb_base <= 0) {
		perror("SMBus Base Address invalid!");
		return -1;
	}
#ifndef LINUX	/* FreeBSD */
  if((iofl=open("/dev/io",000))<0){
	return -1;
  }
#else			/* LINUX */
  iopl(3);
#endif
  return 0;
}
static int CloseIO()
{
#ifndef LINUX	/* FreeBSD */
  close(iofl);
#else			/* LINUX */
  iopl(0);
#endif
}

static WriteByte(int addr,int value)
{
  if(smb_writebyte(smb_base, LM_ADDR, addr, value) == -1) {
	perror("smb_writebyte");
	exit(-1);
  }
}

static int ReadByte(int addr)
{
  int ret;
  ret = smb_readbyte(smb_base, LM_ADDR, addr);
  if(ret == -1) {
/*	perror("smb_readbyte");
	exit(-1); */
	ret = 0xFF;
  }
  ret &= 0xFF;
  return ret;
}

static int ReadTemp1()
{
  int ret;
  ret = smb_readword(smb_base, WBtemp1_ADDR, 0x0);
  if(ret == -1) {
/*	perror("smb_readword");
	exit(-1); */
	ret = 0xFFFF;
  }
  ret &= 0xFFFF;
  return ret;
}
static int ReadTemp2()
{
  int ret;
  ret = smb_readword(smb_base, WBtemp2_ADDR, 0x0);
  if(ret == -1) {
/*	perror("smb_readword");
	exit(-1); */
	ret = 0xFFFF;
  }
  ret &= 0xFFFF;
  return ret;
}

struct lm_methods method_smb={
  OpenIO,CloseIO,ReadByte,WriteByte,ReadTemp1,ReadTemp2
};

#endif
