/************************************************
	Subroutines to get Mother Board Information

	Information related to WinBond W83781D Chip
		and National Semiconductor LM78/LM79/LM75 Chips
	by Alex van Kaam

	Information for VIA VT82C686A/B
	by ":p araffin.(Yoneya)", MANY THANKS!!

	Information for SMBus access
	by Linux lm_sensor homepage, MANY THANKS!!
		http://www.netroedge.com/~lm78/

 ************************************************/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "methods.h"
#include "pci_hwm.h"

#include <stdio.h>

extern int viahwm_base, smb_base;
extern int fahrn_flag;

extern struct lm_methods method_isa;
extern struct lm_methods method_smb;
extern struct lm_methods method_via;
static struct lm_methods *this_method;

#define OpenIO() (this_method->Open());
#define CloseIO() (this_method->Close());
#define ReadByte(addr) (this_method->Read((addr)))
#define WriteByte(addr,val) (this_method->Write((addr),(val)))
#define ReadTemp1() (this_method->ReadTemp1())
#define ReadTemp2() (this_method->ReadTemp2())

/* function declarations */
int RstChip(void);
int chk_smb_chip(int);
int Probe_method(char);
int probe_HWMreg(void);
void via_set(void);
void smb_set(int);
void isa_set(void);
int InitMBInfo(char);
int getTemp(float *, float *, float *);
int getVolt(float *, float *, float *, float *, float *, float *, float *);
void SetFanDiv(int, int, int, int);
int getFanSp(int *, int *, int *);

/* debug flag used in InitMBInfo() and OpenIO() */
extern int debug_flag;

/* monitor chip flags */
int lm_flag = 0;
int winbond_flag = 0;
int asus_flag = 0;
int via_flag = 0;
int ite_flag = 0;

/* temp1/2 flags */
int temp1_flag = 0;
int temp2_flag = 0;

int RstChip(void)
{
  return WriteByte(0x40,0x01);
}


/*----------------------
	Initialization
  ----------------------*/

chk_smb_chip(int chip)
{
	int ret = chip;

	switch (chip) {
		case VIA686HWM:
		case VIA686SMB:
			fprintf(stderr, "VIA686(KT133)");
			break;
		case VIA596SMB:
			fprintf(stderr, "VIA596");
			break;
		case PIIX4SMB:
			fprintf(stderr, "IntelPIIX4(440BX)");
			break;
		case VIA8233SMB:
			fprintf(stderr, "VIA8233(KT266)");
			break;
		case ICH801SMB:
			fprintf(stderr, "Intel801/810/815(ICH/ICH2)");
			break;
		case AMD756SMB:
			fprintf(stderr, "AMD756/766");
			break;
		default:
			ret = 0;
	}
	return ret;
}

Probe_method(char method)
{
	int i, n;

	if (method == 'V') {
  		if ((n = pci_hwm_smb_prob(&viahwm_base, &smb_base)) == VIA686HWM) {
		/* VIA VT82C686 HWM is available */
			this_method = &method_via;
				if (debug_flag > 1) {
					fprintf(stderr, ">>> Testing Reg's at VIA686 HWM <<<\n");
					probe_HWMReg();
				}
			via_set();
			return 0;
		} else if (n != -1) {
			fprintf(stderr, "No VIA686 HWM available!!\n");
			return 1;
		}
	} else if (method == 'S') {
  		if ((n = pci_hwm_smb_prob(&viahwm_base, &smb_base)) > 0) {
		/* There is SMBus PowerManagement, hardware monitor functionality ? */
			this_method = &method_smb;
				if (debug_flag > 1)
					fprintf(stderr, ">>> Testing Reg's at SMBus <<<\n");
			if (probe_HWMReg() >= 5) {
				smb_set(n);
				return 0;
			}
			fprintf(stderr, "No HWM available, though SMBus[");
			chk_smb_chip(n); fprintf(stderr, "] found!!\n");
			return 1;
		} else if (n != -1) {
			fprintf(stderr, "No SMBus HWM available!!\n");
			return 1;
		}
	} else if (method == 'I') {
	/* Just try ISA-IO registers */
		this_method = &method_isa;
				if (debug_flag > 1)
					fprintf(stderr, ">>> Testing Reg's at ISA-IO <<<\n");
		if ((n = probe_HWMReg()) >= 5) {
			isa_set();
			return 0;
		} else if (n != -1) {
			fprintf(stderr, "No ISA-IO HWM available!!\n");
			return 1;
		}
	} else {
	/* No Input Option: Try probing HWM type*/
	  	if ((n = pci_hwm_smb_prob(&viahwm_base, &smb_base)) == VIA686HWM) {
		/* VIA VT82C686 HWM is available */
			this_method = &method_via;
				if (debug_flag > 1) {
					fprintf(stderr, ">>> Testing Reg's at VIA686 HWM <<<\n");
					probe_HWMReg();
				}
			via_set();
			return 0;
		} else if (n > 0) {
		/* There is SMBus PowerManagement, hardware monitor functionality ? */
			this_method = &method_smb;
				if (debug_flag > 1)
					fprintf(stderr, ">>> Testing Reg's at SMBus <<<\n");
			if (probe_HWMReg() >= 5) {
				smb_set(n);
				return 0;
			} else if (debug_flag) {
				fprintf(stderr, "* SMBus["); chk_smb_chip(n);
				fprintf(stderr, "] found, but No HWM available on it!!\n");
			}
			goto isa_chk;
		} else {
isa_chk:	/* Just try ISA-IO registers */
			this_method = &method_isa;
				if (debug_flag > 1)
					fprintf(stderr, ">>> Testing Reg's at ISA-IO <<<\n");
			if ((n = probe_HWMReg()) >= 5) {
				isa_set();
				return 0;
			} else if (n != -1) {
				fprintf(stderr, "No Hardware Monitor found!!\n");
				return 1;
			}
		}
	}
	return -1;
}

/* Register checked for probe_HWMReg() */
int chkReg[] = {
	0x40, 0x41, 0x42, 0x43,
	0x44, 0x47, 0x48, 0x49,
	0x4A, 0x4E, 0x4F, 0x58,
	0x20, 0x22, 0x24, 0x25,
	0x27, 0x29, 0x2A, 0x2B,
	0x0B, 0x0C, 0x0D, 0x0E,
	NULL };

probe_HWMReg()
{
	int i, r, n, ret = 0;

	n=OpenIO(); if(n != 0) return -1;

	for (i = 0; chkReg[i] != NULL; i++) {
		r = chkReg[i];
		n=ReadByte(r);
		if(n != 0xFF)  ++ret;
		if (debug_flag > 1) {
			if ((i + 1) % 4 == 0)
				fprintf(stderr, " CR%02X:0x%02X\n",r , n);
			else
				fprintf(stderr, " CR%02X:0x%02X,",r , n);
		}
	}
	if (debug_flag > 1) fprintf(stderr, "\n");

	CloseIO();
	return ret;
}

void via_set()
{
	via_flag = 1;
	if(debug_flag)
		fprintf(stderr, "Using VIA686 HWM directly!!\n");
}

void smb_set(int chip)
{
	if(debug_flag) {
#if !defined(LINUX) && defined(HAVE_SMBUS) && defined(SMBUS_IOCTL)
		fprintf(stderr, "Using SMBus-ioctl access method[");
#else		/* SMBus direct access routines */
		fprintf(stderr, "Using SMBus access method[");
#endif
		chk_smb_chip(chip);
		fprintf(stderr, "]!!\n");
		if (chip == VIA686HWM || chip == VIA686SMB)
			via_flag = 1;
	}
}

void isa_set()
{
	if(debug_flag)
		fprintf(stderr, "Using ISA-IO access method!!\n");
}

InitMBInfo(char method)
{
  int n, n_d, n_v, n_c, adrs, adrt1, adrt2;

  if ((n = Probe_method(method)) != 0) {
	return n;
  }

  n=OpenIO();
  if (n != 0)
	return -1;

/* debug information */

	n=ReadByte(0x48)&0x7F;
	adrs = 2*n;
	if(debug_flag) printf("  MainSMBusADDR(CR48)  = %2X \n", adrs);

	n=ReadByte(0x4A);
	temp1_flag = (n & 0x08)>>3;
	temp2_flag = (n & 0x80)>>7;
	adrt1 = 2*(0x48 +   (n & 0x07) );
	adrt2 = 2*(0x48 + ( (n & 0x70)>>4) );
	if(debug_flag) printf("  SubSMBusADDRs(CR4A)  = %2X, %2X", adrt1, adrt2);
	if(debug_flag) {
		if(!temp1_flag)
			printf("  [Temp1 exists,");
		else
			printf("  [no Temp1,");
		if(!temp2_flag)
			printf(" Temp2 exists]\n");
		else
			printf(" no Temp2]\n");
	}

	n_d=ReadByte(0x49);
	n=n_d&0xFE;
	if(debug_flag) printf("  DeviceID(CR49&0xFE)  = %2X\n", n);

	WriteByte(0x4E, 0x00);
	n=n_v=ReadByte(0x4F);
	if(debug_flag) printf("  VenderID(CR4E=0,CR4F)= %2X\n", n);
	n=n_c=ReadByte(0x58);
	if(debug_flag) printf("  ChipID(CR58)         = %2X\n", n);

/* probe HWM chip more carefully */

	if (n_d == 0xFF && n_v == 0xFF && n_c == 0xFF) {
		fprintf(stderr, "No Hardware Monitor found!!\n");
		return -1;
	} else if (n_v == 0xA3 ) {
		if (n_c < 0x20) { /* 0x10 (or 0x11) */
			winbond_flag = 1;
			if(debug_flag) printf("* WinBond Chip, W83781D");
		} else if (n_c < 0x30) { /* 0x21 */
			winbond_flag = 2;
			if(debug_flag) printf("* WinBond Chip, W83627HF");
		} else if (n_c < 0x40) { /* 0x30 */
			winbond_flag = 2;
			if(debug_flag) printf("* WinBond Chip, W83782D");
		} else if (n_c < 0x50) { /* 0x40 */
			winbond_flag = 2;
			if(debug_flag) printf("* WinBond Chip, W83783S");
		} else if (n_c < 0x60) {
			winbond_flag = 2;
			if(debug_flag) printf("* WinBond unknown Chip");
		} else if (n_c < 0x70) { /* 0x60 */
			winbond_flag = 2;
			if(debug_flag) printf("* WinBond Chip, W83697HF");
		} else {
			winbond_flag = 2;
			if(debug_flag) printf("* WinBond unknown Chip");
		}
		if(via_flag) via_flag = 0;
	} else if (n_v == 0xC3) {
		asus_flag = 1;
		if(debug_flag) printf("* Asus Chip, AS99127F");
		if(via_flag) via_flag = 0;
	} else if (n_c == 0x90) {
		ite_flag = 1;
		if(debug_flag) printf("* ITE Chip, IT8705F");
		if(via_flag) via_flag = 0;
	} else if ((n_d & 0xFE) == 0x40) {
		lm_flag = 1;
		if(debug_flag) printf("* Nat.Semi.Con. Chip, LM78");
		if(via_flag) via_flag = 0;
	} else if ((n_d & 0xFE) == 0xC0) {
		lm_flag = 1;
		if(debug_flag) printf("* Nat.Semi.Con. Chip, LM79");
		if(via_flag) via_flag = 0;
	} else {
	  if(via_flag) {
		if(debug_flag) printf("* VIA Chip, VT82C686A/B");
	  } else {
		lm_flag = 1;
		if(debug_flag) printf("* Unknown chip (assume LM78/79)");
	  }
	}
		if(debug_flag) printf(" found\n");

	if ( adrs != LM_ADDR) {
		if(debug_flag) printf("  Warning: SMBus address is wrong ??\n");
	}
	if ( !temp1_flag && adrt1 != WBtemp1_ADDR) {
		if(debug_flag) printf("  Warning: Temp1 address is wrong ??\n");
	}
	if ( !temp2_flag && adrt2 != WBtemp2_ADDR) {
		if(debug_flag) printf("  Warning: Temp2 address is wrong ??\n");
	}

	CloseIO();
	return 0;
}


/*-------------------------
	Getting Temperatures
  -------------------------*/

#define traFahrn(x)	((x) * 1.6 + 32.0)

static float via686temp_tab[256]= {\
   .00,   .00,   .00,   .00,   .00,   .00,   .00,   .00,
   .00,   .00,   .00,   .00,   .00,   .00,   .00,-50.00,
-48.00,-46.09,-44.26,-42.52,-40.86,-39.28,-37.78,-36.35,
-34.99,-33.70,-32.47,-31.31,-30.20,-29.15,-28.14,-27.18,
-26.27,-25.39,-24.55,-23.73,-22.95,-22.18,-21.44,-20.70,
-19.98,-19.27,-18.57,-17.87,-17.19,-16.51,-15.85,-15.20,
-14.56,-13.93,-13.32,-12.72,-12.14,-11.58,-11.03,-10.50,
 -9.99, -9.50, -9.03, -8.57, -8.11, -7.67, -7.22, -6.77,
 -6.31, -5.85, -5.37, -4.87, -4.36, -3.81, -3.24, -2.64,
 -2.00, -1.33,  -.65,   .00,   .59,  1.12,  1.60,  2.03,
  2.42,  2.79,  3.12,  3.44,  3.75,  4.05,  4.35,  4.67,
  4.99,  5.34,  5.71,  6.09,  6.49,  6.91,  7.34,  7.78,
  8.23,  8.69,  9.15,  9.62, 10.09, 10.57, 11.04, 11.51,
 11.98, 12.44, 12.90, 13.35, 13.80, 14.24, 14.68, 15.12,
 15.55, 15.99, 16.42, 16.85, 17.27, 17.70, 18.13, 18.56,
 18.99, 19.42, 19.85, 20.29, 20.72, 21.15, 21.59, 22.03,
 22.47, 22.90, 23.34, 23.78, 24.22, 24.66, 25.10, 25.54,
 25.99, 26.43, 26.87, 27.31, 27.75, 28.19, 28.63, 29.07,
 29.51, 29.95, 30.38, 30.82, 31.25, 31.68, 32.11, 32.53,
 32.96, 33.38, 33.80, 34.22, 34.63, 35.05, 35.48, 35.90,
 36.33, 36.76, 37.20, 37.64, 38.09, 38.55, 39.02, 39.50,
 39.99, 40.49, 41.00, 41.52, 42.06, 42.60, 43.15, 43.70,
 44.27, 44.84, 45.41, 45.99, 46.57, 47.16, 47.75, 48.35,
 48.94, 49.54, 50.13, 50.73, 51.33, 51.94, 52.55, 53.16,
 53.78, 54.41, 55.04, 55.68, 56.32, 56.98, 57.64, 58.32,
 59.00, 59.69, 60.40, 61.12, 61.85, 62.60, 63.36, 64.13,
 64.93, 65.73, 66.56, 67.40, 68.26, 69.14, 70.05, 70.97,
 71.91, 72.88, 73.86, 74.88, 75.92, 76.98, 78.08, 79.21,
 80.36, 81.55, 82.78, 84.03, 85.33, 86.66, 88.04, 89.45,
 90.90, 92.40, 93.94, 95.53, 97.16, 98.85,100.58,102.36,
104.19,106.07,108.01,110.00,   .00,   .00,   .00,   .00,
   .00,   .00,   .00,   .00,   .00,   .00,   .00,   .00 };

int getTemp(float *t1, float *t2, float *t3)
{
	int n;

	n = OpenIO();
	if (n != 0)
		return -1;

  if(via_flag) {	/* VIA686's HWM */

	n=ReadByte(0x20);
	*t1 = via686temp_tab[(n & 0xFF)];

	n=ReadByte(0x21);
	*t2 = via686temp_tab[(n & 0xFF)];

	n=ReadByte(0x1F);
	*t3 = via686temp_tab[(n & 0xFF)];

  } else if(ite_flag) {  /* ITE's IT8705F */

	n=ReadByte(0x2A);
	*t1 = n;

	n=ReadByte(0x2B);
	*t2 = n;

	n=ReadByte(0x29);
	*t3 = n;

  } else {	/* other Winbond/LM78 type HWM */

		/* start from main temp */
	n=ReadByte(0x27);
	*t1 = n;
	if (*t1 > 100.0) *t1 = 0.0;

		/* second and/or third temp */
	*t2 = *t3 = 0.0;
	if (!lm_flag) {
	  if(!temp1_flag) {
		n=ReadTemp1();
		*t2 = (n & 0xFF) + ((n & 0xFF00) >> 15)/2.;
		if (*t2 > 100.0) *t2 = 0.0;
	  }
	  if(!temp2_flag) {
		n=ReadTemp2();
		*t3 = (n & 0xFF) + ((n & 0xFF00) >> 15)/2.;
		if (*t3 > 100.0) *t3 = 0.0;
	  }
	}

  }
	CloseIO();
	if(fahrn_flag) {
		if(*t1 != 0.0) *t1 = traFahrn(*t1);
		if(*t2 != 0.0) *t2 = traFahrn(*t2);
		if(*t3 != 0.0) *t3 = traFahrn(*t3);
	}
	return 0;
}

/*--------------------
	Getting Voltages
  --------------------*/

int getVolt(float *vc0, float *vc1, float *v33,\
			float *v50p, float *v50n,\
			float *v12p, float *v12n)
{
	int n;

	n = OpenIO();
	if (n != 0)
		return -1;

  if(via_flag) {	/* VIA686's HWM */

	n=ReadByte(0x22);
	if (n <= 0x40) n |= 0x100;
	*vc0  = (n * 25 + 133) *  5 / 10512.;

	n=ReadByte(0x23) ;
	if (n <= 0x40) n |= 0x100;
	*vc1  = (n * 25 + 133) *  5 / 10512.;

	n=ReadByte(0x24);
	if (n <= 0x40) n |= 0x100;
	*v33  = (n * 25 + 133) *  5 /  7884.;

	n=ReadByte(0x25);
	if (n <= 0x40) n |= 0x100;
	*v50p = (n * 25 + 133) * 26 / 26280.;

	n=ReadByte(0x26) ;
	if (n <= 0x40) n |= 0x100;
	*v12p =	(n * 25 + 133) * 63 / 26280.;

	*v12n = 0.;

	*v50n = 0.;

  } else {	/* other Winbond/LM78 type HWM */

	n=ReadByte(0x20);
	*vc0 = n * 0.016;

	n=ReadByte(0x21);
	*vc1 = n * 0.016;

	n=ReadByte(0x22) ;
	*v33 = n * 0.016;

	n=ReadByte(0x23);
	*v50p = n * 0.016 * 1.68;

	n=ReadByte(0x24);
	if (asus_flag) {
		*v12p = n * 0.016 * 4.028;
	} else {
		*v12p = n * 0.016 * 3.80;
	}

	n=ReadByte(0x25) ;
	if (asus_flag) {
		*v12n = - n * 0.016 * 4.033;
	} else if (winbond_flag == 2) {
		*v12n =  (n * 0.016 - 3.6 * 0.8112) / 0.1888;
	} else {
		*v12n = - n * 0.016 * 3.477;
	}

	n=ReadByte(0x26);
	if (winbond_flag == 2) {
		*v50n =  (n * 0.016 - 3.6 * 0.6818) / 0.3182;
	} else {
		*v50n = - n * 0.016 * 1.505;
	}

  }
	CloseIO();
	return 0;
}

/*----------------------
	Getting Fan Speed
  ----------------------*/

/*
	Controlling Fan Divisor: CR = 0x47.
	Upper 4bits = upper 2bits for fan2, lower 2bits for fan1.

         7       3     0
        +-+-+-+-+-+-+-+-+     xx = 00,01,10,11  div1 = 1,2,4,8
        |y y x x| VolID |     yy = 00,01,10,11  div2 = 1,2,4,8
        +-+-+-+-+-+-+-+-+    initial values: xx=01, yy=01

	In this program, only div1/2 = 2 or 4 are used.
 */

void SetFanDiv(int nd1, int nd2, int n1, int n2)
{
	int reg, reg0;

	if (n1 == 0xFF)
		if (nd1 < 3) ++nd1;
	if (n2 == 0xFF)
		if (nd2 < 3) ++nd2;

	reg0 = ReadByte(0x47);
	reg = (nd1 << 4) | (nd2 << 6) | (reg0 & 0x0F);
	if (reg0 != reg)
		WriteByte(0x47, reg);
}

int getFanSp(int *r1, int *r2, int *r3)
{
	int n, n1, n2, n3, nd1, nd2, div3 = 2;

	n = OpenIO();
	if (n != 0)
		return -1;

  if(ite_flag) {	/* ITE's IT8705F */
	n=ReadByte(0x0B); nd1 = n & 0x07; nd2 = (n >> 3) & 0x07;
	n1=ReadByte(0x0D);
	n2=ReadByte(0x0E);
	n3=ReadByte(0x0F);
  } else {			/* other chips */
	n=ReadByte(0x47); nd1 = (n & 0x30) >> 4; nd2 = (n & 0xC0) >> 6;
	if(via_flag) n1=ReadByte(0x29); else n1=ReadByte(0x28);
	if(via_flag) n2=ReadByte(0x2A); else n2=ReadByte(0x29);
	if(via_flag) n3=0; else n3=ReadByte(0x2A);
  }

	if (n1 == 0 || n1 == 0xFF)
		*r1 = 0;
	else
		*r1 = 1350000 / (n1 * (1 << nd1));

	if (n2 == 0 || n2 == 0xFF)
		*r2 = 0;
	else
		*r2 = 1350000 / (n2 * (1 << nd2));

	if (n3 == 0 || n3 == 0xFF)
		*r3 = 0;
	else
		*r3 = 1350000 / (n3 * div3);

  if(!ite_flag)
	SetFanDiv(nd1, nd2, n1, n2);

	CloseIO();
	return 0;
}
