
/*
 * Yoshifumi R. Shimizu
 *
 * ver.1.00, 1999.02.01-04
 *     Using only Xlib
 *
 * ver.1.01, 1999.02.06
 *     Using X Toolkit
 *
 * ver.1.02, 1999.02.09
 *     Using application resources and option tables
 *     Winbond W83781D supported.
 *     National Semiconductor LM78/79 supported (possibly).
 *
 * ver.1.03, 1999.02.20-23
 *     Adding temp3, and increasing resouces / option tables
 *     -DNO_TEMP3 compiling option for not showing temp3
 *
 * ver.1.04, 1999.05.04
 *     Slight improvement for getting resources
 *
 * ver.1.05, 1999.11.13
 *     Support SMBus access method.  Debugging infomation.
 *     Winbond W83781D, W83782D, W83783S supported.
 *     Asus AS99127F HWM chip supported.
 *
 * ver.1.06, 2001.08.22
 *     New SMBus access method without "intpm" driver!
 *     VIA VT82C686A/B's HWM chip supported.
 *
 * ver.1.07, 2002.01.16
 *     Patch for supporting detailed output for MRTG,
 *     provided by Koji MORITA (morita@cybird.co.jp).
 *     Showing temperture in Fahrenheit.
 *
 *
 * mbmon  --- command-line mother board monitor
 *
 *
 * << Acknowledgements >>
 *
 * Information related to WinBond W83781D LM78/LM79 Chips by Alex van Kaam.
 *  http://mbm.livewiredev.com/
 *
 * Information on VT82C686A/B chips by ":p araffin(Yoneya)".
 *  http://homepage1.nifty.com/paraffin
 *
 * Information for SMBus access by Linux lm_sensor homepage.
 *  http://www.netroedge.com/~lm78/
 *
 */

#ifdef SMALL_MBMON	/* small_mbmon */

#include "mbmon_small.c"

#else			/* ! small_mbmon */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/sysctl.h>
#include <string.h>
#include <sys/param.h>

#define DEFAULT_SEC 5

/* Fahrenheit flag used in getTemp() */
int fahrn_flag = 0;

/* debug flag used in InitMBInfo() */
int debug_flag = 0;

usage(void)
{
	fprintf(stderr, "MotherBoard Monitor, ver.1.07 by YRS\n"\
		"Usage: mbmon [options...] <seconds for sleep> (default %d sec)\n"\
		" options:\n"\
		"  -V|S|I: access method (using \"VIA686 HWM directly\"|\"SMBus\"|\"ISA I/O port\")\n"\
		"  -f: temperature in Fahrenheit\n"\
		"  -d: debug mode (any other option will be ignored)\n"\
		"  -h: print help message(this) and exit\n"
		"  -c count: repeat <count> times and exit\n"\
		"  -T|F [1-7]: print Temperature|Fanspeed according to following styles\n"\
		"	style1: data1\\n\n"\
		"	style2: data2\\n\n"\
		"	style3: data3\\n\n"\
		"	style4: data1\\ndata2\\n\n"\
		"	style5: data1\\ndata3\\n\n"\
		"	style6: data2\\ndata3\\n\n"\
		"	style7: data1\\ndata2\\ndata3\\n\n"\
		"  -u: print system uptime\n"\
		"  -t: print present time\n"\
		"  -n|N: print hostname(long|short style)\n"\
		"  -i: print integers in the summary(with -T option)\n"\
		, DEFAULT_SEC);
	exit(1);
}

uptime(time_t now)
{
#ifndef LINUX	/* FreeBSD */
	struct timeval  boottime;
	time_t uptime;
	int days, hrs, mins, secs;
	int mib[2];
	size_t size;
	char buf[256], tmp[256];

	mib[0] = CTL_KERN;
	mib[1] = KERN_BOOTTIME;
	size = sizeof(boottime);
	if (sysctl(mib, 2, &boottime, &size, NULL, 0) != -1 &&
		boottime.tv_sec != 0) {
		uptime = now - boottime.tv_sec;
		uptime += 30;
		days = uptime / 86400;
		uptime %= 86400;
		hrs = uptime / 3600;
		uptime %= 3600;
		mins = uptime / 60;
		secs = uptime % 60;
		if (days > 0)
			printf("%d day%s,", days, days > 1 ? "s" : "");
		if (hrs > 0 && mins > 0)
			printf(" %2d:%02d", hrs, mins);
		else if (hrs > 0)
			printf(" %d hr%s", hrs, hrs > 1 ? "s" : "");
		else if (mins > 0)
			printf(" %d min%s", mins, mins > 1 ? "s" : "");
		else
			printf(" %d sec%s", secs, secs > 1 ? "s" : "");
		printf("\n");
	}
#endif
}

hostname(int sh_flag)
{
	char *p, hostname[MAXHOSTNAMELEN];

	if (gethostname(hostname, (int)sizeof(hostname)))
		err(1, "gethostname");
	if (sh_flag && (p = strchr(hostname, '.')))
		*p = '\0';
	printf("%s\n", hostname);
}


main(int argc, char *argv[])
{
	int n, sec;
	int count = 0, temperature = 0, fanspeed = 0;
	float temp1 = 0.0, temp2 = 0.0, temp3 = 0.0;
	float vc0, vc1, v33, v50p, v12p, v12n, v50n;
	int rot1, rot2, rot3;
	extern char *optarg;
	extern int optind;
	char *name;
	char *format = "%+", buf[256];
	int ch, method=' ';
	struct tm lt;
	time_t now;
	int time_flag = 0;
	int uptime_flag = 0;
	int hostname_flag = 0;
	int sh_flag = 0;
	int integer_flag = 0;

	name=argv[0];
	while((ch=getopt(argc,argv,"VSIfdDc:T:F:tunNih"))!=-1){
	  switch(ch){
	  case 'V':
		if(method == ' ')
			method='V';
		else
		usage();
		break;
	  case 'S':
		if(method == ' ')
		method='S';
		else
		usage();
		break;
	  case 'I':
		if(method == ' ')
		method='I';
		else
		usage();
		break;
	  case 'f':
		fahrn_flag = 1;
		break;
	  case 'd':
		debug_flag = 1;
		break;
	  case 'D':
		debug_flag = 2;	/* debug option level 2 */
		break;
	  case 'c':
		count = atoi(optarg);
		break;
	  case 'T':
		if(fanspeed != 0)
			usage();
		temperature = atoi(optarg);
		break;
	  case 'F':
		if(temperature != 0)
			usage();
		fanspeed = atoi(optarg);
		break;
	  case 't':
		time_flag = 1;
		break;
	  case 'u':
		uptime_flag = 1;
		break;
	  case 'n':
		if(hostname_flag != 0)
			usage();
		hostname_flag = 1;
		break;
	  case 'N':
		if(hostname_flag != 0)
			usage();
		hostname_flag = 1;
		sh_flag = 1;
		break;
	  case 'i':
		integer_flag = 1;
		break;
	  case 'h':
		usage();
		break;
	  default:
		usage();
	  }
	} 
	argc-=optind;
	argv+=optind;
	if (argc >= 1) {
	  if((n = atoi(argv[0])) > 0) {
		sec = n;
	  } else {
		fprintf(stderr, "  argument (%s) should be integer, stop!\n", argv[0]);
		exit (1);
	  }
	} else {
		sec = DEFAULT_SEC;
	}
	if ((n=InitMBInfo(method))!=0){
		perror("InitMBInfo");
		if(n < 0)
			fprintf(stderr,"This program needs \"setuid root\"!!\n");
		exit (1);
	}
	if (debug_flag)
		exit (0);

	while(1){

/* get temperature */

	getTemp(&temp1, &temp2, &temp3);

/* get fan speeds */

	getFanSp(&rot1, &rot2, &rot3);

/* get voltages */

	getVolt(&vc0, &vc1, &v33, &v50p, &v50n, &v12p, &v12n);

/* print data */

	if(temperature == 0 && fanspeed == 0) {
		printf("Temp.= %4.1f, %4.1f, %4.1f;",temp1, temp2, temp3);
		printf(" Rot.= %4d, %4d, %4d\n", rot1, rot2, rot3);
		printf(" Vcore = %4.2f, %4.2f; Volt. = %4.2f, %4.2f, %5.2f, %6.2f, %5.2f\n", vc0, vc1, v33, v50p, v12p, v12n, v50n);
	} else if(fanspeed == 0) {
		if(integer_flag == 0) {
			if(temperature == 1)
				printf("%4.1f\n", temp1);
			else if(temperature == 2)
				printf("%4.1f\n", temp2);
			else if(temperature == 3)
				printf("%4.1f\n", temp3);
			else if(temperature == 4)
				printf("%4.1f\n%4.1f\n", temp1, temp2);
			else if(temperature == 5)
				printf("%4.1f\n%4.1f\n", temp1, temp3);
			else if(temperature == 6)
				printf("%4.1f\n%4.1f\n", temp2, temp3);
			else if(temperature == 7)
				printf("%4.1f\n%4.1f\n%4.1f\n", temp1, temp2, temp3);
		} else {
			if(temperature == 1)
				printf("%2d\n", (int)(temp1 + 0.5));
			else if(temperature == 2)
				printf("%2d\n", (int)(temp2 + 0.5));
			else if(temperature == 3)
				printf("%2d\n", (int)(temp3 + 0.5));
			else if(temperature == 4)
				printf("%2d\n%2d\n", (int)(temp1 + 0.5), (int)(temp2 + 0.5));
			else if(temperature == 5)
				printf("%2d\n%2d\n", (int)(temp1 + 0.5), (int)(temp3 + 0.5));
			else if(temperature == 6)
				printf("%2d\n%2d\n", (int)(temp2 + 0.5), (int)(temp3 + 0.5));
			else if(temperature == 7)
				printf("%2d\n%2d\n%2d\n", (int)(temp1 + 0.5), (int)(temp2 + 0.5), (int)(temp3 + 0.5));
		}
	} else if(temperature == 0) {
		if(fanspeed == 1)
			printf("%4d\n", rot1);
		else if(fanspeed == 2)
			printf("%4d\n", rot2);
		else if(fanspeed == 3)
			printf("%4d\n", rot3);
		else if(fanspeed == 4)
			printf("%4d\n%4d\n", rot1, rot2);
		else if(fanspeed == 5)
			printf("%4d\n%4d\n", rot1, rot3);
		else if(fanspeed == 6)
			printf("%4d\n%4d\n", rot2, rot3);
		else if(fanspeed == 7)
			printf("%4d\n%4d\n%4d\n", rot1, rot2, rot3);
	}
	now = time(NULL);
	if (time_flag == 1) {
		lt = *localtime(&now);
		strftime(buf, sizeof(buf), format, &lt);
		printf("%s\n", buf);
	}
	if (uptime_flag == 1) {
		uptime(now);
	}
	if (hostname_flag == 1) {
		hostname(sh_flag);
	}

/* count */
	if (count != 0)
	  if (count > 1)
		count--;
	  else
		exit(0);

/* sleep */
	sleep(sec);

	}

}

#endif			/* ! small_mbmon */
