
/*
 * Yoshifumi R. Shimizu
 *
 * ver.1.00, 1999.02.01-04
 *     Using only Xlib
 *
 * ver.1.01, 1999.02.06
 *     Using X Toolkit
 *
 * ver.1.02, 1999.02.09
 *     Using application resources and option tables
 *     Winbond W83781D supported.
 *     National Semiconductor LM78/79 supported (possibly).
 *
 * ver.1.03, 1999.02.20-23
 *     Adding temp3, and increasing resouces / option tables
 *     -DNO_TEMP3 compiling option for not showing temp3
 *
 * ver.1.04, 1999.05.04
 *     Slight improvement for getting resources
 *
 * ver.1.05, 1999.11.13
 *     Support SMBus access method.  Debugging infomation.
 *     Winbond W83781D, W83782D, W83783S supported.
 *     Asus AS99127F HWM chip supported.
 *
 * ver.1.06, 2001.08.22
 *     New SMBus access method without "intpm" driver!
 *     VIA VT82C686A/B's HWM chip supported.
 *
 * mbmon  --- command-line mother board monitor
 *
 *
 * << Acknowledgements >>
 *
 * Information related to WinBond W83781D LM78/LM79 Chips by Alex van Kaam.
 *
 * http://mbm.livewiredev.com/
 *
 * Information on VT82C686A/B chips by ":p araffin(Yoneya)".
 *
 * http://homepage1.nifty.com/paraffin
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <unistd.h>
#define DEFAULT_SEC 5

/* Fahrenheit flag used in getTemp() */
int fahrn_flag = 0;

/* debug flag used in InitMBInfo() */
int debug_flag = 0;

main(int argc, char *argv[])
{
	int n, sec;
	float temp1 = 0.0, temp2 = 0.0, temp3 = 0.0;
	float vc0, vc1, v33, v50p, v12p, v12n, v50n;
	int rot1, rot2, rot3;
	extern char *optarg;
	extern int optind;
	char *name;
	int ch, method=' ';

	name=argv[0];
	while((ch=getopt(argc,argv,"VSIfd"))!=-1){
	  switch(ch){
	  case 'V':
	    method='V';
	    break;
	  case 'S':
	    method='S';
	    break;
	  case 'I':
	    method='I';
	    break;
	  case 'f':
		fahrn_flag = 1;
		break;
	  case 'd':
		debug_flag = 1;
	    break;
	  default:
	    fprintf(stderr, "Usage: %s -[V|S|I] [-d (debug)] <seconds for sleep>"\
		    " (default %d sec)\n", argv[0], DEFAULT_SEC);
	    exit(1);
	  }
	} 
	argc-=optind;
	argv+=optind;
	if (argc >= 1) {
	  if((n = atoi(argv[0])) > 0) {
		sec = n;
	  } else {
	    fprintf(stderr, "  argument (%s) should be integer, stop!\n", argv[0]);
		exit (1);
	  }
	} else {
		sec = DEFAULT_SEC;
	}
	if ((n=InitMBInfo(method))!=0){
		perror("InitMBInfo");
		if(n < 0)
			fprintf(stderr,"This program needs \"setuid root\"!!\n");
		exit (1);
	}
	if (debug_flag)
		exit (0);

	while(1){

/* Temperature */

	getTemp(&temp1, &temp2, &temp3);

	printf("Temp.= %4.1f, %4.1f, %4.1f;",temp1, temp2, temp3);

/* Fan Speeds */

	getFanSp(&rot1, &rot2, &rot3);

	printf(" Rot.= %4d, %4d, %4d\n", rot1, rot2, rot3);

/* Voltages */

	getVolt(&vc0, &vc1, &v33, &v50p, &v50n, &v12p, &v12n);

	printf(" Vcore = %4.2f, %4.2f; Volt. = %4.2f, %4.2f, %5.2f, %6.2f, %5.2f\n", vc0, vc1, v33, v50p, v12p, v12n, v50n);

/* sleep */
	sleep(sec);

	}

}
