
/* Looking for Power Management ChipSet by checking
	the PCI Configuration Register, by YRS 2001.08.

	Information on how to access SMBus is provided
	by ":p araffin.(Yoneya)", MANY THANKS!!

	Information for SMBus access by Linux lm_sensor homepage.
	http://www.netroedge.com/~lm78/
 */

#include "pci_hwm.h"

static int iofl;

int canSMBUS_base[] = {
	getSMBBA1,
	getSMBBA2,
	getSMBBA3,
	getSMBBA4,
	NULL };

static int open_io()
{
#ifndef LINUX	/* FreeBSD */
  if((iofl=open("/dev/io",000))<0){
	return -1;
  }
#else			/* LINUX */
  iopl(3);
	return 0;
#endif
}
static int close_io()
{
#ifndef LINUX	/* FreeBSD */
  close(iofl);
#else			/* LINUX */
  iopl(0);
#endif
}

chk_port_byte(int addr)
{
	int ret;
	if(open_io() == -1) return -1;
	ret = inb((u_short) addr);
	close_io();
	return ret;
}

pci_hwm_smb_prob(int *hwm_base, int *smb_base)
{
	u_int iofl, dat;
	u_short dev, fun;
	int ret = 0, inq_smbba;

	if(open_io() == -1) return -1;
	*hwm_base = *smb_base = 0;
	for (dev = 0; dev < PCI_DEVM; ++dev) {
	  for (fun = 0; fun < PCI_FUNM; ++fun) {
		switch (pci_conf_read(PCI_BUSN, dev, fun, 0x00)) {
		case ID_VIA686:
			dat = pci_conf_read(PCI_BUSN, dev, fun, VIA686HWM_prob);
			if (dat && 0x00000001) {
				*hwm_base = 0xFFFE & \
					pci_conf_read(PCI_BUSN, dev, fun, VIA686HWM_base);
				ret = VIA686HWM;
			} else {
				ret = VIA686SMB;
			}
			inq_smbba = canSMBUS_base[0];
			break;
		case ID_VIA596:
		case ID_VIA596B:
			ret = VIA596SMB;
			inq_smbba = canSMBUS_base[0];
			break;
		case ID_PIIX4:
			ret = PIIX4SMB;
			inq_smbba = canSMBUS_base[0];
			break;
		case ID_VIA8233:
			ret = VIA8233SMB;
			inq_smbba = canSMBUS_base[1];
			break;
		case ID_I801AA:
		case ID_I801AB:
		case ID_I801BA:
		case ID_I801CA:
			ret = ICH801SMB;
			inq_smbba = canSMBUS_base[2];
			break;
		case ID_AMD756:
		case ID_AMD766:
			ret = AMD756SMB;
			inq_smbba = canSMBUS_base[3];
			break;
		default:
		}
		if (ret) {
			*smb_base = 0xFFFE & \
				pci_conf_read(PCI_BUSN, dev, fun, inq_smbba);
			goto ending;
		}
	  }
	}
ending:
	close_io();
	return ret;
}

pci_smb_prob(int smb_base[], u_int chip_id[])
{
	u_int iofl, chip, dat, addr;
	u_short dev, fun;
	int i, n = 0;

	if(open_io() == -1) return -1;
	for (dev = 0; dev < PCI_DEVM; ++dev) {
	  for (fun = 0; fun < PCI_FUNM; ++fun) {
		chip = pci_conf_read(PCI_BUSN, dev, fun, 0x00);
		if (chip != 0xFFFFFFFF) {
		  for (i = 0; (addr = canSMBUS_base[i]) != NULL; i++) {
			dat = pci_conf_read(PCI_BUSN, dev, fun, addr);
#ifdef PCI_LIST
	if (i == 0) {
		printf("bus=0x00:dev=0x%02X:fun=0x%02X ---> ", dev, fun);
		printf("chip=0x%08lX [0x%02X] SMBase=0x%08lX\n", chip, addr, dat);
	} else {
		printf("                                                ");
		printf("[0x%02X] SMBase=0x%08lX\n", addr, dat);
	}
#endif
			if (dat != 0x0 && dat != 0xFFFFFFFF \
				&& (dat & 0x0F) == 1 && (dat & 0xFFF00000) == 0) {
				smb_base[n] = (dat & 0xFFFE);
				chip_id[n] = chip;
				if (inb(smb_base[n]) != 0xFF)
					++n;
			}
		  }
		}
	  }
	}
	close_io();
	return n;
}

/* PCI Configuration Register

 31              23              15              7             0
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
|1|0 0 0 0 0 0 0|  Bus Num      |Dev Num  |Func |Reg Num    |0 0|
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

 */

u_int pci_conf_read(u_short bus, u_short dev, u_short fun, u_short reg)
{
/* NOTE: do not check length of "bus, dev, fun, reg"!! */
/*		reg = (Reg Num) << 2  */
	u_int pix = 0, dat;
	pix |= reg;
	pix |= (fun << 8);
	pix |= (dev << 11);
	pix |= (bus << 16);
	pix |= 0x80000000; 
	OUTl((unsigned int) PCI_CFIO, pix); WAIT;
	dat = inl((unsigned int) PCI_CREAD); WAIT;
	pix &= 0x7FFFFFFF; 
	OUTl((unsigned int) PCI_CFIO, pix); WAIT;
	return dat;
}
