
/* SMB handling routines, by YRS 2001.08.
	Information on how to access SMBus is provided
	by ":p araffin.(Yoneya)", MANY THANKS!!

	SMBus IO method:

	smb_base  : Base Address
	addr      : Slave Device Address
	cmd       : Command Register 

	Note that SMBus Slave Device Address is totall 1byte data,
	whose upper 7bits is for address and the lowest bit for read (=1)
	and for write (=0).
	The input "addr" in the following routines is this 1byte data,
	where the lowest bit can be either 0 or 1.

            7             0
           +-+-+-+-+-+-+-+-+
           | Slave addr. |f|    f = 0 for write, =1 for read.
           +-+-+-+-+-+-+-+-+

	Do not confuse "Slave address" which is "addr(here)"/2!

 */

#include "smb_io.h"

int smb_readbyte(int smb_base, int addr, int cmd)
{
	int dat,i, saddr = 2*(addr/2);

top:
	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		OUTb((u_short) smb_base, (char) dat); WAIT; WAIT;
		if (!(dat & 0x09))
			goto step;
		if (!(dat & 0x01))
			break;
	}
	OUTb((u_short) (smb_base + 2), 0x02); WAIT; WAIT;

step:
	OUTb((u_short) (smb_base + 4), (char) (saddr + 1)); WAIT; WAIT;
	OUTb((u_short) (smb_base + 3), (char) cmd); WAIT; WAIT;
	OUTb((u_short) (smb_base + 2), 0x48); WAIT; WAIT;

	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		if ((dat & 0x08))
			goto top;
		if ((dat & 0x06))
			break;
	}
	if ((dat & 0x02)) {
		dat = inb((u_short) (smb_base + 5)); WAIT; WAIT;
		return (dat & 0xFF);
	} else
		return -1;
}

int smb_readword(int smb_base, int addr, int cmd)
{
	int dat,i, saddr = 2*(addr/2);

top:
	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		OUTb((u_short) smb_base, (char) dat); WAIT; WAIT;
		if (!(dat & 0x09))
			goto step;
		if (!(dat & 0x01))
			break;
	}
	OUTb((u_short) (smb_base + 2), 0x02); WAIT; WAIT;

step:
	OUTb((u_short) (smb_base + 4), (char) (saddr + 1)); WAIT; WAIT;
	OUTb((u_short) (smb_base + 3), (char) cmd); WAIT; WAIT;
	OUTb((u_short) (smb_base + 2), 0x4C); WAIT; WAIT;

	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		if ((dat & 0x08))
			goto top;
		if ((dat & 0x06))
			break;
	}
	if ((dat & 0x02)) {
		i = inb((u_short) (smb_base + 6)); WAIT; WAIT;
		dat = inb((u_short) (smb_base + 5)); WAIT; WAIT;
		return (((i << 8) + dat) & 0xFFFF);
	} else
		return -1;
}

int smb_writebyte(int smb_base, int addr, int cmd, int value)
{
	int dat,i, saddr = 2*(addr/2);

top:
	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		OUTb((u_short) smb_base, (char) dat); WAIT; WAIT;
		if (!(dat & 0x09))
			goto step;
		if (!(dat & 0x01))
			break;
	}
	OUTb((u_short) (smb_base + 2), 0x02); WAIT; WAIT;

step:
	OUTb((u_short) (smb_base + 4), (char) saddr); WAIT; WAIT;
	OUTb((u_short) (smb_base + 3), (char) cmd); WAIT; WAIT;
	OUTb((u_short) (smb_base + 5), (char) value); WAIT; WAIT;
	OUTb((u_short) (smb_base + 2), 0x48); WAIT; WAIT;

	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		if ((dat & 0x06))
			break;
	}
	if ((dat & 0x02))
		return 0;
	else
		return -1;
}

int smb_writeword(int smb_base, int addr, int cmd, int value)
{
	int dat,i, saddr = 2*(addr/2);

top:
	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		OUTb((u_short) smb_base, (char) dat); WAIT; WAIT;
		if (!(dat & 0x09))
			goto step;
		if (!(dat & 0x01))
			break;
	}
	OUTb((u_short) (smb_base + 2), 0x02); WAIT; WAIT;

step:
	OUTb((u_short) (smb_base + 4), (char) saddr); WAIT; WAIT;
	OUTb((u_short) (smb_base + 3), (char) cmd); WAIT; WAIT;
	OUTb((u_short) (smb_base + 5), (char) (value & 0xFF)); WAIT; WAIT;
	OUTb((u_short) (smb_base + 6), (char) (value >> 8)); WAIT; WAIT;
	OUTb((u_short) (smb_base + 2), 0x4C); WAIT; WAIT;

	for (i = 0; i < LOOP_COUNT; ++i) {
		dat = inb((u_short) smb_base); WAIT; WAIT;
		if ((dat & 0x06))
			break;
	}
	if ((dat & 0x02))
		return 0;
	else
		return -1;
}
